/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConfEditorService.java 5084 2009-08-28 09:03:19Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.eos.confeditor.service;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.ow2.jasmine.monitoring.eos.common.JasmineEventManager;
import org.ow2.jasmine.monitoring.eos.common.jms.JMSConnection;
import org.ow2.jasmine.monitoring.eos.common.jms.event.JasmineMessageEvent;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

public class ConfEditorService implements Serializable {

    /**
     *  serialVersionUID.
     */
    private static final long serialVersionUID = -1254916345191626735L;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(ConfEditorService.class);

    /**
     * The JMS connection instance.
     */
    private JMSConnection connection = null;

    /**
     * The event manager instance.
     */
    private JasmineEventManager manager = null;

    /**
     * A Timer to prevent from disconnection.
     */
    private Timer timer = null;

    // 2 minutes
    private static long TIMER = 120000;

    public ConfEditorService() {
        this.manager = new JasmineEventManager();
    }

    public void alert() {
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
        System.out.println("alerted or not");
    }

    /**
     * Load an xml file to configure graphs.
     *
     * @param file the xml file to load
     * @return a string representing the xml file, without the header.
     */
    public String loadFile(final String file) {
        String result = "";
        FileReader f = null;
        BufferedReader buffer = null;
        try {

            String line = "";

            f = new FileReader(file);

            buffer = new BufferedReader(f);

            boolean write = false;

            while ((line = buffer.readLine()) != null) {
                if (line.contains("<monitor")) {
                    write = true;
                }
                if (write) {
                    result += line + "\n";
                }
            }

            return result;

        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } finally {
            try {
                buffer.close();
                f.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return "";

    }
    /**
     * connect to the JMS topic.
     * @return -1 if an error occured, 0 otherwise.
     */
    public int connectTopic() {
        System.out.println("connected to ConfEditorService.java");
        System.out.println("connected to ConfEditorService.java");
        System.out.println("connected to ConfEditorService.java");
        System.out.println("connected to ConfEditorService.java");
        System.out.println("connected to ConfEditorService.java");
        System.out.println("connected to ConfEditorService.java");
        System.out.println("connected to ConfEditorService.java");
        if (this.connection == null) {
            this.connection = new JMSConnection(this.manager);
        }

        // set the timer
        this.timer = new Timer();
        this.timer.schedule(new DisconnectTask(), ConfEditorService.TIMER);

        return this.connection.connect();
    }

    /**
     * Disconnect from the JMS topic.
     * @return -1 if an error occurred, 0 otherwise.
     */
    public int disconnectTopic() {
        logger.debug("Disconnecting from the JMS topic...");
        try {

            if (this.connection == null) {
                return -1;
            }
            return this.connection.disconnect();
        } finally {
            this.timer.cancel();

        }

    }

    /**
     * Get all the messages in the queue.
     * @return a list of events.
     */
    public List<JasmineMessageEvent> getMessage() {

        this.resetTimer();

        List<JasmineMessageEvent> list = new ArrayList<JasmineMessageEvent>();

        list.addAll(this.manager.getAll());

        return list;
    }

    /**
     * Reset the timer.
     */
    private void resetTimer() {
        this.timer.cancel();
        this.timer = new Timer();
        this.timer.schedule(new DisconnectTask(), ConfEditorService.TIMER);
    }

    /**
     * Implementation of TimerTask.
     * @author Guillaume Renault
     *
     */
    class DisconnectTask extends TimerTask {

        @Override
        public void run() {
            disconnectTopic();

        }

    }

}
